@@ -9,7 +9,6 @@ use fbas_analyzer::*;
 use std::collections::HashMap;
 use std::path::Path;
 
-use hex;
 use sha3::{Digest, Sha3_256};
 
 pub fn main() {
@@ -23,9 +22,14 @@ pub fn main() {
     let inactive_nodes = FilteredNodes::from_json_file(nodes_json, |v| v["active"] == false);
     let fbas = fbas.without_nodes_pretty(&inactive_nodes.into_pretty_vec());
 
+    // This reduces the FBAS to its "core" - the union of all quorum-containing strongly connected
+    // components (usually there is just one). Splitting sets analyses performend on this object
+    // will return only such splitting sets that can cause two or more core nodes to diverge from
+    // each other.
+    let fbas = fbas.to_core();
+
     // This changes the order of nodes (sorts them by public key) and hence renumbers all IDs!
-    // This also discard all nodes that are not part of a strongly connected component, as they
-    // are irrelevant for analysis.
+    // This also discards all unsatisfiable nodes as they are irrelevant for analysis.
     let fbas = fbas.to_standard_form();
 
     // Now we can calculate a hash of the FBAS such that, if two FBASs have the same hash, their
@@ -116,6 +120,7 @@ fn do_analysis(fbas: &Fbas) -> CustomResultsStruct {
 }
 
 #[derive(Debug, Clone)]
+#[allow(dead_code)]
 struct CustomResultsStruct {
     minimal_blocking_sets: NodeIdSetVecResult,
     minimal_splitting_sets: NodeIdSetVecResult,
