@@ -5,6 +5,7 @@ use fbas_analyzer::*;
 use quicli::prelude::*;
 use structopt::StructOpt;
 
+use itertools::Itertools;
 use std::path::PathBuf;
 
 /// Learn things about a given FBAS (parses data from stellarbeat.org)
@@ -26,7 +27,14 @@ struct Cli {
     #[structopt(short = "s", long = "minimal-splitting-sets")]
     minimal_splitting_sets: bool,
 
-    /// Output (and find) all minimal quorums, minimal blocking sets and minimal splitting sets.
+    /// For each found minimal splitting set, output two or more quorums that it splits, i.e., two
+    /// or more quorums that don't intersect after the nodes in the splitting set are deleted from
+    /// the FBAS.
+    #[structopt(short = "S", long = "minimal-splitting-sets-with-affected-quorums")]
+    minimal_splitting_sets_with_affected_quorums: bool,
+
+    /// Output (and find) all minimal quorums, minimal blocking sets and minimal splitting sets,
+    /// i.e., the same as `-qbs`.
     #[structopt(short = "a", long = "all")]
     all: bool,
 
@@ -79,6 +87,13 @@ struct Cli {
     #[structopt(long = "ignore-one-node-quorums")]
     ignore_one_node_quorums: bool,
 
+    /// Shrink the FBAS to its core nodes prior to analysis, i.e., to the union of all quorum-containing strongly
+    /// connected components. Splitting sets analyses will miss any splitting sets that do not
+    /// consist entirely of core nodes and don't cause at least one pair of core nodes to end up in
+    /// non-intersecting quorums.
+    #[structopt(long = "only-core-nodes")]
+    only_core_nodes: bool,
+
     #[structopt(flatten)]
     verbosity: Verbosity,
 }
@@ -102,13 +117,13 @@ fn main() -> CliResult {
     } else {
         None
     };
-    let analysis = Analysis::new(&fbas);
+    let analysis = init_analysis(&fbas, args.only_core_nodes);
 
-    let (q, b, s) = extract_main_todos(&args);
+    let (q, b, s, big_s) = extract_main_todos(&args);
     let output = Output::init(&args, &fbas, &groupings);
 
     report_overview(&analysis, &groupings, &output);
-    output.comment_newline();
+    output.optional_newline();
 
     find_and_report_symmetric_clusters(&analysis, &groupings, &output);
 
@@ -127,10 +142,15 @@ fn main() -> CliResult {
     if b {
         find_and_report_minimal_blocking_sets(&analysis, &groupings, &output);
     }
-    if s {
+    if s || big_s {
         find_and_report_minimal_splitting_sets(&analysis, &groupings, &output);
     }
-    if q || b || s {
+    if big_s {
+        find_and_report_minimal_splitting_sets_with_affected_quorums(
+            &analysis, &groupings, &output,
+        );
+    }
+    if q || b {
         report_top_tier_uncondensed(&analysis, &groupings, &output);
     }
     Ok(())
@@ -141,7 +161,7 @@ fn load_fbas(
     ignore_inactive_nodes: bool,
     ignore_one_node_quorums: bool,
 ) -> Fbas {
-    let fbas = if let Some(nodes_path) = o_nodes_path {
+    let mut fbas = if let Some(nodes_path) = o_nodes_path {
         eprintln!("Reading FBAS JSON from file...");
         let mut fbas = Fbas::from_json_file(nodes_path);
         if ignore_inactive_nodes {
@@ -149,20 +169,20 @@ fn load_fbas(
                 FilteredNodes::from_json_file(nodes_path, |v| v["active"] == false);
             fbas = fbas.without_nodes_pretty(&inactive_nodes.into_pretty_vec());
         }
-        if ignore_one_node_quorums {
-            fbas = fbas.without_nodes(&fbas.one_node_quorums());
-        }
         fbas
     } else {
         eprintln!("Reading FBAS JSON from STDIN...");
-        if ignore_inactive_nodes || ignore_one_node_quorums {
+        if ignore_inactive_nodes {
             panic!(
-                "Ignoring nodes is currently not supported when reading an FBAS from STDIN;
-                perhaps filter the input yourself? (e.g., with `jq`)"
+                "Ignoring nodes based on additional JSON fields is currently not supported when
+                reading an FBAS from STDIN; perhaps filter the input yourself? (e.g., with `jq`)"
             );
         }
         Fbas::from_json_stdin()
     };
+    if ignore_one_node_quorums {
+        fbas = fbas.without_nodes(&fbas.one_node_quorums());
+    }
     eprintln!("Loaded FBAS with {} nodes.", fbas.number_of_nodes());
     fbas
 }
@@ -204,14 +224,27 @@ fn maybe_load_countries<'a>(
         None
     }
 }
-fn extract_main_todos(args: &Cli) -> (bool, bool, bool) {
+fn init_analysis(fbas: &Fbas, only_core_nodes: bool) -> Analysis {
+    let mut analysis = Analysis::new(fbas);
+    if only_core_nodes {
+        analysis.shrink_to_core_nodes();
+    }
+    analysis
+}
+fn extract_main_todos(args: &Cli) -> (bool, bool, bool, bool) {
     if args.all {
-        (true, true, true)
+        (
+            true,
+            true,
+            true,
+            args.minimal_splitting_sets_with_affected_quorums,
+        )
     } else {
         (
             args.minimal_quorums,
             args.minimal_blocking_sets,
             args.minimal_splitting_sets,
+            args.minimal_splitting_sets_with_affected_quorums,
         )
     }
 }
@@ -242,8 +275,8 @@ macro_rules! do_time_and_report {
 macro_rules! do_time_maybe_merge_and_report {
     ($result_name:expr, $operation:expr, $groupings:expr, $output:expr) => {{
         let (mut result, duration) = timed!($operation);
-        if let Some(ref orgs) = $groupings {
-            result = result.merged_by_group(orgs).minimal_sets();
+        if let Some(ref groups) = $groupings {
+            result = result.merged_by_group(groups).minimal_sets();
         }
         $output.timed_result($result_name, result, duration);
     }};
@@ -251,12 +284,12 @@ macro_rules! do_time_maybe_merge_and_report {
 
 fn report_overview(analysis: &Analysis, groupings: &Option<Groupings>, output: &Output) {
     output.result("nodes_total", analysis.all_nodes().len());
-    if let Some(ref orgs) = groupings {
+    if let Some(ref groups) = groupings {
         output.result(
             "nodes_total_merged",
-            analysis.all_nodes().merged_by_group(orgs).len(),
+            analysis.all_nodes().merged_by_group(groups).len(),
         );
-        output.comment("(Nodes belonging to the same grouping will be counted as one.)");
+        output.optional_comment("(Nodes belonging to the same grouping will be counted as one.)");
     }
 }
 fn check_and_report_if_has_quorum_intersection(
@@ -265,7 +298,7 @@ fn check_and_report_if_has_quorum_intersection(
     alternative_check: bool,
 ) {
     let has_quorum_intersection = if alternative_check {
-        output.comment("Alternative quorum intersection check...");
+        output.optional_comment("Alternative quorum intersection check...");
         let ((has_quorum_intersection, quorums), duration) =
             timed!(analysis.has_quorum_intersection_via_alternative_check());
         output.timed_result("has_quorum_intersection", has_quorum_intersection, duration);
@@ -282,9 +315,9 @@ fn check_and_report_if_has_quorum_intersection(
         analysis.has_quorum_intersection() // from cache
     };
     if has_quorum_intersection {
-        output.comment("\nAll quorums intersect 👍\n");
+        output.optional_comment("\nAll quorums intersect 👍\n");
     } else {
-        output.comment(
+        output.optional_comment(
             "\nSome quorums don't intersect 👎 Safety severely threatened for some nodes!\n\
                  (Also, the remaining results here might not make much sense.)\n",
         );
@@ -299,14 +332,14 @@ fn find_and_report_symmetric_clusters(
     output_uncondensed.describe = false;
     do_time_and_report!(
         "symmetric_clusters",
-        if let Some(ref orgs) = groupings {
-            orgs.merge_quorum_sets(analysis.symmetric_clusters())
+        if let Some(ref groups) = groupings {
+            groups.merge_quorum_sets(analysis.symmetric_clusters())
         } else {
             analysis.symmetric_clusters()
         },
         output_uncondensed
     );
-    output.comment_newline();
+    output.optional_newline();
 }
 fn find_and_report_minimal_quorums(
     analysis: &Analysis,
@@ -319,7 +352,7 @@ fn find_and_report_minimal_quorums(
         groupings,
         output
     );
-    output.comment(&format!(
+    output.optional_comment(&format!(
         "\nWe found {} minimal quorums.\n",
         analysis.minimal_quorums().len()
     ));
@@ -335,7 +368,7 @@ fn find_and_report_minimal_blocking_sets(
         groupings,
         output
     );
-    output.comment(&format!(
+    output.optional_comment(&format!(
         "\nWe found {} minimal blocking sets (minimal indispensable sets for global liveness). \
             Control over any of these sets is sufficient to compromise the liveness of all nodes \
             and to censor future transactions.\n",
@@ -353,7 +386,7 @@ fn find_and_report_minimal_splitting_sets(
         groupings,
         output
     );
-    output.comment(&format!(
+    output.optional_comment(&format!(
         "\nWe found {} minimal splitting sets \
              (minimal indispensable sets for safety). \
              Control over any of these sets is sufficient to compromise safety by \
@@ -361,22 +394,41 @@ fn find_and_report_minimal_splitting_sets(
         analysis.minimal_splitting_sets().len()
     ));
 }
+fn find_and_report_minimal_splitting_sets_with_affected_quorums(
+    analysis: &Analysis,
+    groupings: &Option<Groupings>,
+    output: &Output,
+) {
+    let mut results = analysis.minimal_splitting_sets_with_affected_quorums();
+    if let Some(ref groups) = groupings {
+        results = results
+            .into_iter()
+            .map(|(mut key, mut result)| {
+                key = key.merged_by_group(groups);
+                result = result.merged_by_group(groups).minimal_sets();
+                (key, result)
+            })
+            .unique()
+            .collect();
+    }
+    output.keyed_results_uncondensed("minimal_splitting_sets_with_affected_quorums", results);
+}
 fn report_top_tier_uncondensed(
     analysis: &Analysis,
     groupings: &Option<Groupings>,
     output: &Output,
 ) {
     let mut top_tier = analysis.top_tier();
-    if let Some(ref orgs) = groupings {
-        top_tier = top_tier.merged_by_group(orgs);
+    if let Some(ref groups) = groupings {
+        top_tier = top_tier.merged_by_group(groups);
     }
     output.result_uncondensed("top_tier", top_tier.clone());
-    output.comment(
-        &format!(
-            "\nThere is a total of {} distinct nodes involved in all of these sets (this is the \"top tier\").\n",
-            top_tier.len()
-        )
-    );
+    output.optional_comment(&format!(
+        "\nThere is a total of {} distinct nodes involved in all \
+            minimal quorums and minimal blocking sets. \
+            This is the \"top tier\".\n",
+        top_tier.len()
+    ));
 }
 
 #[derive(Clone)]
@@ -415,12 +467,12 @@ impl<'a> Output<'a> {
             groupings,
         }
     }
-    fn comment(&self, comment: &str) {
+    fn optional_comment(&self, comment: &str) {
         if !self.results_only {
             println!("{}", comment);
         }
     }
-    fn comment_newline(&self) {
+    fn optional_newline(&self) {
         if !self.results_only {
             println!();
         }
@@ -439,18 +491,38 @@ impl<'a> Output<'a> {
         );
     }
     fn result(&self, result_name: &str, result: impl AnalysisResult) {
-        if self.describe {
-            println!("{}: {}", result_name, result.into_describe_string());
-        } else {
-            self.result_uncondensed(result_name, result);
-        }
+        println!("{}: {}", result_name, self.make_string(result));
     }
     fn result_uncondensed(&self, result_name: &str, result: impl AnalysisResult) {
-        let result_string = if self.output_pretty {
+        println!("{}: {}", result_name, self.make_string_uncondensed(result));
+    }
+    fn keyed_results_uncondensed(
+        &self,
+        result_name: &str,
+        results: impl IntoIterator<Item = (impl AnalysisResult, impl AnalysisResult)>,
+    ) {
+        println!("{}:", result_name);
+        for (key, result) in results.into_iter() {
+            println!(
+                "- {}: {}",
+                self.make_string_uncondensed(key),
+                self.make_string_uncondensed(result)
+            );
+        }
+        println!();
+    }
+    fn make_string(&self, result: impl AnalysisResult) -> String {
+        if self.describe {
+            result.into_describe_string()
+        } else {
+            self.make_string_uncondensed(result)
+        }
+    }
+    fn make_string_uncondensed(&self, result: impl AnalysisResult) -> String {
+        if self.output_pretty {
             result.into_pretty_string(self.fbas, self.groupings.as_ref())
         } else {
             result.into_id_string()
-        };
-        println!("{}: {}", result_name, result_string);
+        }
     }
 }
