@@ -62,7 +62,7 @@ impl Fbas {
     pub fn new_generic_unconfigured(n: usize) -> Self {
         let mut fbas = Fbas::new();
         for _ in 0..n {
-            fbas.add_generic_node(QuorumSet::new());
+            fbas.add_generic_node(QuorumSet::new_empty());
         }
         fbas
     }
@@ -107,7 +107,7 @@ impl Fbas {
         !node_set.is_empty()
             && node_set
                 .iter()
-                .all(|x| self.nodes[x].is_quorum_slice(node_set))
+                .all(|x| self.nodes[x].quorum_set.is_quorum_slice(node_set))
     }
 }
 impl Hash for Fbas {
@@ -132,21 +132,21 @@ impl PartialEq for Fbas {
     }
 }
 
-#[derive(Clone, Eq, PartialEq, Ord, PartialOrd, Hash, Debug, Default)]
+#[derive(Clone, Eq, PartialEq, Ord, PartialOrd, Hash, Debug)]
 pub struct Node {
     pub(crate) public_key: PublicKey,
     pub(crate) quorum_set: QuorumSet,
 }
 impl Node {
-    pub fn new(public_key: PublicKey) -> Self {
-        let quorum_set = QuorumSet::new();
+    /// Returns a node with an empty quorum set that induces one-node quorums!
+    pub fn new_unconfigured(public_key: PublicKey) -> Self {
         Node {
             public_key,
-            quorum_set,
+            quorum_set: QuorumSet::new_empty(),
         }
     }
-    pub fn is_quorum_slice(&self, node_set: &NodeIdSet) -> bool {
-        self.quorum_set.is_quorum_slice(node_set)
+    pub fn is_quorum_slice(&self, own_id: NodeId, node_set: &NodeIdSet) -> bool {
+        node_set.contains(own_id) && self.quorum_set.is_quorum_slice(node_set)
     }
 }
 
@@ -159,25 +159,14 @@ mod tests {
     use super::*;
     use std::path::Path;
 
-    #[test]
-    fn new_node() {
-        let node = Node::new("test".to_string());
-        assert_eq!(node.public_key, "test");
-        assert_eq!(
-            node.quorum_set,
-            QuorumSet {
-                threshold: 0,
-                validators: vec![],
-                inner_quorum_sets: vec![]
-            }
-        );
-    }
-
     #[test]
     #[should_panic]
     fn add_node_panics_on_duplicate_public_key() {
         let mut fbas = Fbas::new();
-        let node = Node::new("test".to_string());
+        let node = Node {
+            public_key: "test".to_string(),
+            quorum_set: QuorumSet::new_empty(),
+        };
         fbas.add_node(node.clone());
         fbas.add_node(node);
     }
