@@ -8,14 +8,15 @@ pub(crate) struct RawFbas(pub(crate) Vec<RawNode>);
 #[serde(rename_all = "camelCase")]
 pub(crate) struct RawNode {
     pub(crate) public_key: PublicKey,
-    #[serde(default)]
+    // If no quorum set is given, we assume that the node is unsatisfiable, i.e., broken.
+    #[serde(default = "RawQuorumSet::new_unsatisfiable")]
     pub(crate) quorum_set: RawQuorumSet,
     #[serde(skip_serializing_if = "Option::is_none")]
     pub(crate) isp: Option<String>,
     #[serde(skip_serializing_if = "Option::is_none")]
     pub(crate) geo_data: Option<RawGeoData>,
 }
-#[derive(Debug, Serialize, Deserialize, Default, PartialEq)]
+#[derive(Debug, Serialize, Deserialize, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub(crate) struct RawQuorumSet {
     pub(crate) threshold: u64,
@@ -23,6 +24,15 @@ pub(crate) struct RawQuorumSet {
     #[serde(default, skip_serializing_if = "Vec::is_empty")]
     pub(crate) inner_quorum_sets: Vec<RawQuorumSet>,
 }
+impl RawQuorumSet {
+    fn new_unsatisfiable() -> Self {
+        Self {
+            threshold: 1,
+            validators: vec![],
+            inner_quorum_sets: vec![],
+        }
+    }
+}
 #[serde_as]
 #[derive(Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
@@ -212,10 +222,10 @@ mod tests {
                 validators: vec![0, 1, 2].into_iter().collect(),
                 inner_quorum_sets: vec![],
             },
-            Default::default(),
+            QuorumSet::new_unsatisfiable(),
         ];
 
-        let actual = Fbas::from_json_str(&input);
+        let actual = Fbas::from_json_str(input);
         let actual_public_keys: Vec<PublicKey> =
             actual.nodes.iter().map(|x| x.public_key.clone()).collect();
         let actual_quorum_sets: Vec<QuorumSet> =
@@ -249,10 +259,10 @@ mod tests {
                 validators: vec![0, 1].into_iter().collect(),
                 inner_quorum_sets: Default::default(),
             },
-            Default::default(),
+            QuorumSet::new(vec![], vec![], 1),
         ];
 
-        let actual = Fbas::from_json_str(&input);
+        let actual = Fbas::from_json_str(input);
         let actual_quorum_sets: Vec<QuorumSet> =
             actual.nodes.into_iter().map(|x| x.quorum_set).collect();
 
@@ -275,7 +285,7 @@ mod tests {
                 "publicKey": "GABMK"
             }]"#;
 
-        let fbas = Fbas::from_json_str(&input);
+        let fbas = Fbas::from_json_str(input);
 
         let expected = vec!["GCGB2", "GCM6Q", "GABMK"];
         let actual: Vec<PublicKey> = fbas.nodes.into_iter().map(|x| x.public_key).collect();
